@@ -85,6 +85,14 @@ def _pygame_color(c: Color) -> pygame.Color:
     return pygame.Color(r, g, b)
 
 
+def _ccw(a,b,c):
+    return (c[1]-a[1]) * (b[0]-a[0]) > (b[1]-a[1]) * (c[0]-a[0])
+
+
+def _intersect(a, b, c, d):
+    return _ccw(a, c, d) != _ccw(b, c, d) and _ccw(a, b, c) != _ccw(a, b, d)
+
+
 # -----------------------------------------------------------------------
 
 # Private functions to scale and factor X and Y values.
@@ -98,6 +106,10 @@ def _scale_y(y: float) -> float:
     return _canvas_height * (_ymax - y) / (_ymax - _ymin)
 
 
+def _scale_point(p: Sequence[float]) -> Sequence[float]:
+    return (_scale_x(p[0]), _scale_y(p[1]))
+
+
 def _factor_x(w: float) -> float:
     return w * _canvas_width / abs(_xmax - _xmin)
 
@@ -163,24 +175,25 @@ def set_canvas_size(w: float = _DEFAULT_CANVAS_SIZE, h: float = _DEFAULT_CANVAS_
 
 def get_canvas_width() -> float:
     """
-    Return the width of the current canvas
+    Return the width of the current canvas.
     """
     return abs(_xmax - _xmin)
 
 
 def get_canvas_height() -> float:
     """
-    Return the height of the current canvas
+    Return the height of the current canvas.
     """
     return abs(_ymax - _ymin)
 
 
-def get_pixel_color(x: float, y: float) -> pygame.Color:
+def get_pixel_color(x: float, y: float) -> Color:
     """
-    Return the color of the pixel at the given user coordinates
+    Return the color of the pixel at the given user coordinates.
     """
     _make_sure_window_created()
-    return _surface.get_at((int(_scale_x(x)), int(_scale_y(y))))
+    c = _surface.get_at((int(_scale_x(x)), int(_scale_y(y))))
+    return Color(c[0], c[1], c[2])
 
 
 def set_x_scale(min: float = _DEFAULT_XMIN, max: float = _DEFAULT_XMAX):
@@ -215,12 +228,11 @@ def set_y_scale(min: float = _DEFAULT_YMIN, max: float = _DEFAULT_YMAX):
     _ymax = max + _BORDER * size
 
 
-def set_scale():
-    set_x_scale()
-    set_y_scale()
-
-
 def set_scale(min: float, max: float):
+    """
+    Set the x-scale and y-scale of the canvas to the same range for
+    both directions.
+    """
     set_x_scale(min, max)
     set_y_scale(min, max)
 
@@ -251,8 +263,8 @@ def set_pen_color(c: Color = _DEFAULT_PEN_COLOR):
 
 def set_pen_color_rgb(r: int = 255, g: int = 255, b: int = 255):
     """
-    Set the pen color to c, where c is an object of class Color.
-    c defaults to dudraw.BLACK.
+    Set the pen color using red, green, and blue values.
+    Defaults to black.
     """
     c = Color(r, g, b)
     global _pen_color
@@ -555,8 +567,15 @@ def polyline(x: Sequence[float], y: Sequence[float]):
             ba = pygame.math.Vector2(a[0] - b[0], a[1] - b[1]).normalize()
             w = ba.rotate(90).normalize()
             w.scale_to_length(line_width/2.0)
-            inner_points.append((b[0] + w.x, b[1] + w.y))
-            outer_points.append((b[0] - w.x, b[1] - w.y))
+            if _intersect(inner_points[-1],
+                          (b[0] + w.x, b[1] + w.y), 
+                          outer_points[-1], 
+                          (b[0] - w.x, b[1] - w.y)):
+                outer_points.append((b[0] + w.x, b[1] + w.y))
+                inner_points.append((b[0] - w.x, b[1] - w.y))
+            else:
+                inner_points.append((b[0] + w.x, b[1] + w.y))
+                outer_points.append((b[0] - w.x, b[1] - w.y))
         else:
             ba = pygame.math.Vector2(a[0] - b[0], a[1] - b[1]).normalize()
             bc = pygame.math.Vector2(c[0] - b[0], c[1] - b[1]).normalize()
@@ -571,23 +590,13 @@ def polyline(x: Sequence[float], y: Sequence[float]):
                 outer_points.append((b[0] + ba.x + bc.x, b[1] + ba.y + bc.y))
                 inner_points.append((b[0] - ba.x - bc.x, b[1] - ba.y - bc.y))
 
-    x = [p[0] for p in inner_points]
-    y = [p[1] for p in inner_points]
-    x.extend([p[0] for p in reversed(outer_points)])
-    y.extend([p[1] for p in reversed(outer_points)])    
-
-    # Scale X and Y values.
-    x_scaled = []
-    for xi in x:
-        x_scaled.append(_scale_x(float(xi)))
-    y_scaled = []
-    for yi in y:
-        y_scaled.append(_scale_y(float(yi)))
-    points = []
-    for i in range(len(x)):
-        points.append((x_scaled[i], y_scaled[i]))
-    points.append((x_scaled[0], y_scaled[0]))
-    pygame.draw.polygon(_surface, _pygame_color(_pen_color), points, 0)
+    for i in range(len(inner_points)-1):
+        points = (
+            _scale_point(inner_points[i]),
+            _scale_point(inner_points[i+1]),
+            _scale_point(outer_points[i+1]),
+            _scale_point(outer_points[i]))
+        pygame.draw.polygon(_surface, _pygame_color(_pen_color), points, 0)
 
 
 def polygon(x: Sequence[float], y: Sequence[float]):
@@ -618,27 +627,13 @@ def polygon(x: Sequence[float], y: Sequence[float]):
             outer_points.append((b[0] + ba.x + bc.x, b[1] + ba.y + bc.y))
             inner_points.append((b[0] - ba.x - bc.x, b[1] - ba.y - bc.y))
 
-    x = [p[0] for p in inner_points]
-    y = [p[1] for p in inner_points]
-    x.append(inner_points[0][0])
-    y.append(inner_points[0][1])
-    x.append(outer_points[0][0])
-    y.append(outer_points[0][1])
-    x.extend([p[0] for p in reversed(outer_points)])
-    y.extend([p[1] for p in reversed(outer_points)])    
-
-    # Scale X and Y values.
-    x_scaled = []
-    for xi in x:
-        x_scaled.append(_scale_x(float(xi)))
-    y_scaled = []
-    for yi in y:
-        y_scaled.append(_scale_y(float(yi)))
-    points = []
-    for i in range(len(x)):
-        points.append((x_scaled[i], y_scaled[i]))
-    points.append((x_scaled[0], y_scaled[0]))
-    pygame.draw.polygon(_surface, _pygame_color(_pen_color), points, 0)
+    for i in range(-1, len(inner_points)-1):
+        points = (
+            _scale_point(inner_points[i]),
+            _scale_point(inner_points[i+1]),
+            _scale_point(outer_points[i+1]),
+            _scale_point(outer_points[i]))
+        pygame.draw.polygon(_surface, _pygame_color(_pen_color), points, 0)
 
 
 def filled_polygon(x: Sequence[float], y: Sequence[float]):
@@ -964,10 +959,10 @@ def text(x: float, y: float, s: str):
     _surface.blit(text, textpos)
 
 
-def picture(pic_path: str, x: float = None, y: float = None):
+def picture(filepath: str, x: float = None, y: float = None):
     """
-    Draw pic on the background canvas centered at (x, y).  pic is an
-    object of class picture.Picture. x and y default to the midpoint
+    Draw pic on the background canvas centered at (x, y). Filepath
+    is specified as a string, and (x, y) defaults to the midpoint
     of the background canvas.
     """
     global _surface
@@ -981,7 +976,7 @@ def picture(pic_path: str, x: float = None, y: float = None):
     y = float(y)
     xs = _scale_x(x)
     ys = _scale_y(y)
-    pic = pygame.image.load(pic_path)
+    pic = pygame.image.load(filepath)
     ws = pic.get_width()
     hs = pic.get_height()
     _surface.blit(pic, [xs - ws / 2.0, ys - hs / 2.0, ws, hs])
@@ -998,22 +993,22 @@ def clear(c: Color = WHITE):
 
 def clear_rgb(r: float = 255, g: float = 255, b: float = 255):
     """
-    Clear the background canvas to color c, where c is an
-    object of class Color. c defaults to dudraw.WHITE.
+    Clear the background canvas to color defined by
+    r, g, and b. Defaults to white (255, 255, 255).
     """
     c = Color(r, g, b)
     _make_sure_window_created()
     _surface.fill(_pygame_color(c))
 
 
-def save(f: str):
+def save(filepath: str):
     """
-    Save the window canvas to file f.
+    Save the window canvas as a .jpg to filepath specified. 
     """
     _make_sure_window_created()
-    if not f.lower().endswith(".jpg"):
-        f += ".jpg"
-    pygame.image.save(_surface, f)
+    if not filepath.lower().endswith(".jpg"):
+        filepath += ".jpg"
+    pygame.image.save(_surface, filepath)
 
 
 # -----------------------------------------------------------------------
@@ -1044,7 +1039,7 @@ def _show_and_wait_forever():
 def show(msec: float = 0.0):
     """
     Copy the background canvas to the window canvas, and
-    then wait for msec milliseconds. msec defaults to infinity.
+    then wait for msec milliseconds. msec defaults to 0.0.
     """
     if msec == float("inf"):
         _show_and_wait_forever()
